#!/bin/sh

. /etc/PG.conf

CODEDIR="/usr/ramdisk/authcode"
FLOWEYE="/usr/ramdisk/bin/floweye"
PAINF="/usr/ramdisk/etc/panabit.inf"

PALANGCONF="${PGETC}/palang.conf"
SYSNAMECONF="${PGETC}/sysname.conf"

Auth_type_file="${PGETC}/App/opser/auth_type.conf"
if [ -f ${Auth_type_file} ]; then
	. ${Auth_type_file}
fi

if [ -f ${PGETC}/ifadmin.conf ]; then
	. ${PGETC}/ifadmin.conf
fi

WEB_LOGGER()
{
	local logfile
	local curtime

	if [ -e /usr/ramdisk/datalog/keepme ]; then
		localfile="/usr/ramdisk/datalog/web_`date +%Y.%m.%d`.log"
	else
		localfile="${DATAPATH}/web_`date +%Y.%m.%d`.log"
	fi

	curtime=`date +%Y.%m.%d/%H:%M:%S`
	echo "${curtime} ${REMOTE_ADDR} ${CGI_username} $1 $2" >> ${localfile}
}


retjson()
{
	printf "Content-Type:application/json;charset=gb2312\r\n"
	printf "\r\n"

	if [ "${3}" = "" ]; then
		echo "{ \"code\": ${1}, \"msg\": \"${2}\", \"data\": \"\"}"
	else
		echo "{ \"code\": ${1}, \"msg\": \"${2}\", \"data\": ${3}}"
	fi

	exit 0
}


getlang()
{
	PALANG="ch"
	SYSNAME=""
	
	[ -f ${PAINF} ] && . ${PAINF}
	[ -f ${PALANGCONF} ] && . ${PALANGCONF}
	[ -f ${SYSNAMECONF} ] && . ${SYSNAMECONF}
	[ "${JOS_RELEASE}" = "2" ] && CHONLY=1
	[ "${ENONLY}" = "" ] && ENONLY=0
	
	data="{\"lang\":\"${PALANG}\",\"enonly\":\"${ENONLY}\",\"chonly\":\"${CHONLY}\",\"title\":\"${SYSNAME}\",\"auth_type\":\"${auth_type}\"}"
	retjson 0 "OK" ${data}
}


codemake()
{
	mkdir -p ${CODEDIR}
	code=`/bin/uuidgen | cut -c5-8`
	echo "CODE=${code}" > ${CODEDIR}/${REMOTE_ADDR}
}


codepost()
{
	[ ! -f ${CODEDIR}/${REMOTE_ADDR} ] && codemake

	. ${CODEDIR}/${REMOTE_ADDR}
	
	retjson 0 "OK" "\"${CODE}\""
}


codecmp()
{
	codefile=${CODEDIR}/${REMOTE_ADDR}

	[ ! -f ${codefile} ] && return
	
	if [ "${CGI_code}" = "" ]; then
		codemake
		retjson 1 "Code_Error"
	fi

	. ${codefile}

	if [ "${CGI_code}" != "${CODE}" ]; then
		codemake
		retjson 1 "Code_Error"
	fi

	[ -f ${codefile} ] && rm -f ${codefile}
}


radius_auth()
{
	RADCLIENT=/usr/ramdisk/app/opser/bin/radclient

	if [ ! -x ${RADCLIENT} ]; then
		CGI_password=`/sbin/md5 -s ${CGI_password} | cut -d'=' -f2 | sed 's/ //g'`
		local_auth
		return
	fi
	
	if [ "${raas_url}" != "" ]; then
		CGI_username="${CGI_username}@${raas_url}"
	fi
	
	${RADCLIENT} -u ${CGI_username} -p ${CGI_password} -r ${rad_ip} -i ${ADMIN_IP} -n ${nas_id} -s ${rad_key} -f ${REMOTE_ADDR} -t ${rad_timeout}
}


local_auth()
{

	if [ ! -f ${PGPATH}/admin/.htpasswd ]; then
		echo "Auth_Fail"
		return
	fi
	
	for line in `cat ${PGPATH}/admin/.htpasswd`
	do
		name=`echo ${line} | cut -d':' -f1`
		if [ "${name}" = "${CGI_username}" ]; then
			pwd=`echo ${line} | cut -d':' -f2-`
			md5pwd=`/sbin/md5 -s ${pwd} | cut -d'=' -f2 | sed 's/ //g'`
			if [ "${md5pwd}" = "${CGI_password}" ]; then
				echo "AUTH_OK"
				return
			fi
		fi
	done

	echo "Auth_Fail"
}


user_login()
{
	# check allow ip list
	if [ -f /etc/pawebhosts ]; then
		exist=`grep -R "^${REMOTE_ADDR}$" /etc/pawebhosts`
		if [ "${exist}" = "" ]; then
			codemake
			retjson 1 "IPDENY"
		fi
	fi

	codecmp

	# local or radius auth
	if [ "${auth_type}" = "radius" ]; then
		auth_ret=`radius_auth`
	elif [ "${auth_type}" = "rad_then_local" ]; then
		auth_ret=`radius_auth`
		if [ "${auth_ret}" = "TIME_OUT" ]; then
			CGI_password=`/sbin/md5 -s ${CGI_password} | cut -d'=' -f2 | sed 's/ //g'`
			auth_ret=`local_auth`
		fi
	else
		auth_ret=`local_auth`
	fi

	# retjson to web
	if [ "${auth_ret}" != "AUTH_OK" ]; then
		codemake
		errmsg=`${FLOWEYE} logger alert "${CGI_username} login FAIL from ${REMOTE_ADDR}"`
		retjson 1 "${auth_ret}"
	fi

	logintime=`date +%s1`
	mkdir -p /usr/ramdisk/weblogin/
	echo "PALANG=${CGI_palang}" > ${PALANGCONF}
	ckinfo="paonline_${CGI_username}_$$_${logintime}"
	ckfile="/usr/ramdisk/weblogin/${ckinfo}"

	echo "ip=${REMOTE_ADDR}" > ${ckfile}
	echo "user=${CGI_username}" >> ${ckfile}
	echo "logintime=`date +%Y.%m.%d/%H:%M:%S`" >> ${ckfile}

	LOG_MSG="û¼"
	[ "${PALANG}" = "en" ] && LOG_MSG="USER_LOGIN"
	WEB_LOGGER "${LOG_MSG}" "user=${CGI_username},ip=${REMOTE_ADDR}"
	errmsg=`${FLOWEYE} logger alert "${CGI_username} login success from ${REMOTE_ADDR}"`

	printf "Content-Type:application/json;charset=gb2312\r\n"
	printf "Set-Cookie: ${PANABIT_CKNAME}=${ckinfo};Secure=true;Path=/;HTTPOnly\r\n"
	printf "\r\n"

	echo "{ \"code\": 0, \"msg\": \"Auth_OK\"}"

	exit 0
}


# action from login.htm
case "${CGI_action}" in
	"getlang")
		getlang
		;;

	"getauthcode")
		codemake
		codepost
		;;
	
	"user_login")
		user_login
		;;
esac


# Login From Cloud or Get userverify.cgi
printf "Content-type: text/html\r\n";
printf "Cache-Control: no-cache\r\n"

if [ "${CGI_pakey}" != "" -a "${CGI_username}" = "admin" ]; then
	keyexist=`${FLOWEYE} cloud list | grep "${CGI_pakey}"`
	if [ "${keyexist}" = "" ]; then
		printf "\r\n"
		echo "<html xmlns:v=\"urn:schemas-microsoft-com:vml\" xmlns:o=\"urn:schemas-microsoft-com:office:office\">"
		echo "<head>"
		echo "<script language=javascript>"
		echo "alert(\"!\");"
		echo "</script>"
		echo "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=gb2312\">"
		echo "<META HTTP-EQUIV=REFRESH CONTENT=\"0;URL=/login/login.htm\">"
		echo "</head>"
		echo "</html>"

		exit 0
	else
		logintime=`date +%s1`
		mkdir -p /usr/ramdisk/weblogin/
		ckinfo="paonline_${CGI_username}_$$_${logintime}"
		ckfile="/usr/ramdisk/weblogin/${ckinfo}"
		errmsg=`${FLOWEYE} cloud remove key=${CGI_pakey}`
		echo "ip=${REMOTE_ADDR}" > ${ckfile}
		echo "user=${CGI_username}" >> ${ckfile}
		echo "logintime=`date +%Y.%m.%d/%H:%M:%S`" >> ${ckfile}

		printf "Set-Cookie: ${PANABIT_CKNAME}=${ckinfo};Path=/;HTTPOnly;palang=${PALANG}\r\n"
		printf "\r\n"

		echo "<html xmlns:v=\"urn:schemas-microsoft-com:vml\" xmlns:o=\"urn:schemas-microsoft-com:office:office\">"
		echo "<head>"
		echo "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=gb2312\">"
		echo "<META HTTP-EQUIV=REFRESH CONTENT=\"0;URL=/index.htm\">"
		echo "</head>"
		echo "</html>"

		exit 0
	fi
fi

printf "\r\n"
echo "<html xmlns:v=\"urn:schemas-microsoft-com:vml\" xmlns:o=\"urn:schemas-microsoft-com:office:office\">"
echo "<head>"
echo "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=gb2312\">"
echo "<META HTTP-EQUIV=REFRESH CONTENT=\"0;URL=/login/index.htm\">"
echo "</head>"
echo "</html>"
	